##############################################################################
#   Copyright (c) 2018 Eynes/E-MIPS (www.eynes.com.ar)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
##############################################################################

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class AllowedOrganizationalUnit(models.Model):
    _name = 'allowed.organizational.unit'
    _rec_name = 'org_unit_id'

    _sql_constraints = [
        ('org_unit_id_unique',
         'UNIQUE(org_unit_id)',
         _("There is another organizational "\
           "unit that overlaps with the current "\
           "organizational unit")),
    ]

    @api.constrains('org_unit_id')
    def _check_duplicate(self):
        org_unit_obj = self.env['organizational.unit']
        for reg in self:
            allowed_org_units = self.search([
                ('org_unit_id', '=', reg.org_unit_id.id),
                ('id', '!=', reg.id)
            ])
            if allowed_org_units:
                msg = _("There is another organizational "\
                        "unit that overlaps with the current "\
                        "organizational unit")
                raise ValidationError(msg)

    org_unit_id = fields.Many2one(
        comodel_name='organizational.unit',
        string='Allowed Organizarional Unit'
    )

    @api.multi
    def compute_employees(self):
        for reg in self:
            employee_obj = self.env['hr.employee']
            employees = employee_obj.search(['|',
                ('allowed_org_unit_id', '=', reg.id),
                ('organizational_unit', '=', reg.org_unit_id.id)
            ])
            employees._compute_allowed_org_unit()
        return True

    @api.multi
    def write(self, vals):
        res = super(AllowedOrganizationalUnit, self).write(vals)
        self.compute_employees()
        return res

    @api.model
    def create(self, values):
        reg = super(AllowedOrganizationalUnit, self).create(values)
        reg.compute_employees()
        return reg
