##############################################################################
#   Copyright (c) 2018 Eynes/E-MIPS (www.eynes.com.ar)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
##############################################################################

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError

class HrCategory(models.Model):
    _name = 'hr.category'
    _order = 'priority asc'

    @api.constrains('priority')
    def _check_priority(self):
        for reg in self:
            if reg.priority < 0:
                raise ValidationError(_("Negative priorities are not allowed"))
            regs = self.search([
                ('id', '!=', reg.id),
                ('priority', '=', reg.priority),
            ])
            if regs:
                raise ValidationError(_(
                    "There is already a category with this priority"
                ))

    @api.constrains('name')
    def _check_name(self):
        for reg in self:
            regs = self.search([
                ('id', '!=', reg.id),
                '|',
                ('name', '=', reg.name),
                ('code', '=', reg.code),
            ])
            if regs:
                raise ValidationError(
                        _("There is other category with same name or code"))


    name = fields.Char(string='Name', required=True)
    code = fields.Char(string='Code', required=True)
    rate_ids = fields.One2many('hr.category.rate', 'category_id', string='Rates')
    priority = fields.Integer(string="Priority")

    @api.multi
    def write(self, values):
        if 'name' in values:
            values['name'] = values['name'].upper()

        return super(HrCategory, self).write(values)

    @api.model
    def create(self, values):
        if 'name' in values:
            values['name'] = values['name'].upper()

        return super(HrCategory, self).create(values)

    @api.multi
    def get_rate(self, date):
        self.ensure_one()
        rate = self.search([
            ('date_from', '<=', date)
            ('category_id', '=', self.id)
        ], order="desc", limit=1)
        return rate.amount

class HrCategoryRate(models.Model):
    _name = 'hr.category.rate'
    _order = 'date_from desc'

    @api.constrains('date_from')
    def check_date_from(self):
        for reg in self:
            regs = self.search([
                ('date_from', '=', reg.date_from),
                ('category_id', '=', reg.category_id.id),
                ('id', '!=', reg.id)])
            if regs:
                raise ValidationError(_("There is other rate with same date"))

    @api.constrains('amount')
    def check_amount(self):
        for reg in self:
            if reg.amount < 0.0:
                raise ValidationError(
                        _("You cannot have a negative rate"))

    @api.depends('period_id')
    def _compute_date_from(self):
        for rate in self:
            rate.date_from = rate.period_id.date_from

    category_id = fields.Many2one('hr.category', string='Category')
    desc = fields.Char(string='Description', required=True)
    amount = fields.Float(string='Amount')
    date_from = fields.Date(
        string='Date From',
        compute=_compute_date_from,
        store=True,
    )
    period_id = fields.Many2one(
        string='Period',
        comodel_name='date.period',
        required=True
    )
