##############################################################################
#   Copyright (c) 2018 Eynes/E-MIPS (www.eynes.com.ar)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
##############################################################################

from odoo import _, api, fields, models
from odoo.exceptions import ValidationError


class HrEmployee(models.Model):
    _name = 'hr.employee'
    _inherit = 'hr.employee'
    _sql_constraints = [('employee_file_id_unique', 'unique(employee_file_id)', _("There is other employee with same file ID"))]

    employee_file_id = fields.Char('File ID', size=32, required=True)
    position = fields.Many2one(
        comodel_name='hr.position',
        string='Position',
        required=True
    )
    organizational_unit = fields.Many2one(
        comodel_name='organizational.unit',
        string='Organizational Unit',
        required=True
    )
    allowed_org_unit_id = fields.Many2one(
        comodel_name='allowed.organizational.unit',
        string='Allowed Organizational Unit',
        compute='_compute_allowed_org_unit',
        store=True
    )
    physical_center = fields.Char('Physical center')
    ceco_id = fields.Many2one(
        comodel_name='account.analytic.account',
        string='CECO',
    )
    ceco_recupero_id = fields.Many2one(
        comodel_name='account.analytic.account',
        string='CECO Recupero',
    )
    category_id = fields.Many2one('hr.category', string='Category')
    reviewed = fields.Boolean(string='Reviewed')
    create_month = fields.Integer(
        string='Create month',
        compute='_compute_create_month',
        store=True
    )
    create_period = fields.Many2one(
        string='Create Period',
        comodel_name='date.period',
        compute='_compute_create_period',
        store=True
    )


    @api.depends('create_date')
    def _compute_create_month(self):
        for employee in self:
            month = employee.create_date.month
            employee.create_month = month

    @api.depends('create_date')
    def _compute_create_period(self):
        period_obj = self.env['date.period']
        for employee in self:
            date = employee.create_date
            period = period_obj.search([
                ('date_from', '<=', date),
                ('date_to', '>=', date)
            ], limit=1)
            employee.create_period = period

    @api.depends('organizational_unit')
    def _compute_allowed_org_unit(self):
        awd_org_obj = self.env['allowed.organizational.unit']
        for employee in self:
            org_unit = employee.organizational_unit
            awd_org = awd_org_obj.search([
                ('org_unit_id', '=', org_unit.id)
            ], limit=1)
            employee.allowed_org_unit_id = awd_org

    @api.constrains('employee_file_id')
    def _check_employee_file_id(self):
        for employee in self:
            regs = self.with_context(active_test=False).search([
                ('id', '!=', employee.id),
                ('employee_file_id', '!=', False),
                ('employee_file_id', '=', employee.employee_file_id),
            ])
            if regs:
                raise ValidationError(
                        _("There is other employee with same file ID"))

    @api.multi
    def set_reviewed(self):
        for employee in self:
            employee.reviewed = not employee.reviewed
