##############################################################################
#   Copyright (c) 2018 Eynes/E-MIPS (www.eynes.com.ar)
#   License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).
##############################################################################

from odoo import api, models


class ResUsers(models.Model):
    _inherit = 'res.users'

    @api.multi
    def notify_user(self, template, model=False, res_ids=False, wo_data=False,
                    msg_vals={}, force_send=False, send_after_commit=True,
                    model_description=False, mail_auto_delete=True):
        """
        @param mail_template template: the template to render
        @param bool wo_data: if template has not ${expr}
        @param str model: model name of the document record
          this mail is related to.
        @param list res_ids: list of ids of document records
          those mails are related to.
        @param dict msg_vals: dictionary of values used to create the message.
          If given it is used instead of accessing ``self`` to lesen query
          count in some simple cases where no notification is actually required
        @param bool force_send: tells whether to send notification emails
          within the current transaction or to use the email queue
        @param bool send_after_commit: if force_send, tells whether
          to send emails after the transaction has been committed
          using a post-commit hook
        @param str model_description: optional data used in notification
          process (see notification templates)
        @param bool mail_auto_delete: delete notification emails once sent
        """
        mail_obj = self.env['mail.message']
        for user in self:
            template = template.sudo().with_context(lang=user.lang)
            partners = user.mapped('partner_id')
            msg_vals = {
                'partner_ids': [(4, pid) for pid in partners.ids],
                'subtype_id': self.env.ref('mail.mt_comment').id,
                'author_id': self.env.ref('base.partner_root').id,
            }
            body_html = template.body_html
            body_html = body_html.replace('\n', '')
            if wo_data:
                msg_vals['subject'] = template.subject
                msg_vals['body'] = body_html
            else:
                body = template._render_template(
                    body_html, model, res_ids)
                subject = template._render_template(
                    template.subject, model, res_ids)
                msg_vals['subject'] = subject
                msg_vals['body'] = body
            mail = mail_obj.sudo().create(msg_vals)
            mail._notify(
                mail, msg_vals,
                force_send=force_send,
                send_after_commit=send_after_commit,
                model_description=model_description,
                mail_auto_delete=mail_auto_delete,
            )
            partners.sudo()._notify_by_chat(mail)
