import pytest
from odoo.exceptions import ValidationError


class TestOrganizationalUnit:
    def test_organizational_unit_repeat(self, env, OU):
        OU.create({
            'name': 'organizational_unit1',
            'parent_id': None
        })
        with pytest.raises(
            ValidationError, match=r".*is other Organizational Unit.*"
        ):
            OU.create({
                'name': 'organizational_unit1',
                'parent_id': None
            })

    def test_create_if_it_does_not_exist_1(self, env, OU):
        OU.find_organizational_unit(['organizational_unit1'])
        organizational_unit_ids = OU.search(
            [
                ('name', '=', 'organizational_unit1'),
                ('parent_id', '=', None)
            ]
        )
        assert organizational_unit_ids

    def test_create_if_it_does_not_exist_3(self, env, OU):
        res = OU.find_organizational_unit(
            [
                'organizational_unit1',
                'organizational_unit2',
                'organizational_unit3',
            ]
        )
        assert res.complete_name == 'organizational_unit1 / ' \
                                    'organizational_unit2 / ' \
                                    'organizational_unit3'

    def test_create_with_record_that_exists(self, env, OU):
        OU.create({
            'name': 'organizational_unit1',
            'parent_id': None
        })
        res = OU.find_organizational_unit(
            [
                'organizational_unit1',
                'organizational_unit2',
                'organizational_unit3',
            ]
        )
        assert res.complete_name == 'organizational_unit1 / ' \
                                    'organizational_unit2 / ' \
                                    'organizational_unit3'

    def test_return_value_with_all_existing_records(self, env, OU):
        first_id = OU.create({
            'name': 'organizational_unit1',
            'parent_id': None
        })
        second_id = OU.create({
            'name': 'organizational_unit2',
            'parent_id': first_id.id
        })
        third_id = OU.create({
            'name': 'organizational_unit3',
            'parent_id': second_id.id
        })
        res = OU.find_organizational_unit(
            [
                'organizational_unit1',
                'organizational_unit2',
                'organizational_unit3',
            ]
        )
        assert third_id.id == res.id

    def test_parent_recursion(self, env, OU):
        ou = OU.create({
            'name': 'organizational_unit1',
            'parent_id': None
        })
        with pytest.raises(
            ValidationError, match=r".*cannot create recursive categories.*"
        ):
            ou.write({
                'name': 'organizational_unit1',
                'parent_id': ou.id
            })
